package fr.ippon.kafkaLatency.streaming;

import com.fasterxml.jackson.databind.ObjectMapper;
import fr.ippon.kafkaLatency.model.Person;
import fr.ippon.kafkaLatency.utils.Config;
import fr.ippon.kafkaLatency.utils.UtilStats;
import org.apache.log4j.Logger;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.Trigger;
import scala.Tuple2;

import java.io.Serializable;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.concurrent.TimeUnit;

/**
 * Ippon Tech 2018.
 */
public class KafkaContinuousSpark implements Serializable {

    private static final ObjectMapper mapper = new ObjectMapper();
    private final static Logger logger = Logger.getLogger(KafkaContinuousSpark.class);

    public static void main(String[] args) {

        StreamingQuery query = null;
        SparkSession spark = null;

        try {
            String brokers = Config.getProperty("bootstrap.servers");
            String topic = Config.getProperty("topic");
            Integer SparkContinuousInterval = Config.getIntProperty("spark.continuous.interval");
            Integer maxPollRecords = Config.getIntProperty("max.poll.records");
            Integer numPartitions = Config.getIntProperty("numPartitions");


            UtilStats stats = new UtilStats("KafkaContinuousSpark");

            // !!! number of kafka partitions must not exceed number of cores
            spark = SparkSession.builder()
                    .appName("KafkaContinuousSpark")
                    .master("local[*]")
                    .getOrCreate();

            Dataset<Tuple2<String, Timestamp>> source = spark.readStream().format("kafka")
                    .option("kafka.bootstrap.servers", brokers)
                    .option("subscribe", topic)
                    .option("max.poll.records", maxPollRecords)
                    .option("startingOffsets", "latest")
                    .load()
                    .select("value", "timestamp")
                    .as(Encoders.tuple(Encoders.STRING(), Encoders.TIMESTAMP()));

            Dataset<Person> ds = source.map((MapFunction<Tuple2<String, Timestamp>, Person>) t -> {
                LocalDateTime now = LocalDateTime.now();

                LocalDateTime parsedDate = t._2.toLocalDateTime();

                long millis = ChronoUnit.MILLIS.between(parsedDate, now);

                Person p = mapper.readValue(t._1, Person.class);

                stats.treatStats(millis);

                return p;
            }, Encoders.bean(Person.class));


            query = ds.writeStream()
                    .outputMode(OutputMode.Append())
                    .format("memory").queryName("test")
                    .trigger(Trigger.Continuous(SparkContinuousInterval, TimeUnit.MILLISECONDS))
                    .start();

            query.awaitTermination();

        } catch (Exception e) {
            logger.error("Error " + e.getMessage());

            assert query != null;
            if (query.isActive()) {
                query.stop();
                spark.stop();
            }

        }

    }

}